<?php

/**
 * Extends the SoapClient class providing:
 * - custom security header, WS-Security
 * - extract SOAP envelope from multipart message
 *
 * See more at http://stackoverflow.com/questions/953639/connecting-to-ws-security-protected-web-service-with-php
 */
class WsSoapClient extends SoapClient {
  private static $WssNamespace = 'http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd';

  private $login;
  private $password;

  /**
   * Constructor.
   */
  public function __construct($wsdl, $options = array()) {
    // Keep authentication for the WS-Security header
    $this->login = $options['login'];
    unset($options['login']);
    $this->password = $options['password'];
    unset($options['password']);
    
    // Forward constructor
    parent::__construct($wsdl, $options);

    // Set the initial Soap header (will be overrided next)
    $objHeader = new SoapHeader('http://docs.oasisopen.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd', 'Security', '', FALSE);
    $this->__setSoapHeaders(array($objHeader));
  }

  /**
   * Overrided to provide custom security header.
   */
  public function __doRequest($request, $location, $action, $version) {
    $domRequest = new DOMDocument();
    $domRequest->loadXML($request);
    $xp = new DOMXPath($domRequest);
    $header = $xp->query('/SOAP-ENV:Envelope/SOAP-ENV:Header')->item(0); 

    // Implement WS-Security
    if ($this->login) {
      $securityToken = $domRequest->createElementNS(self::$WssNamespace, 'wsse:Security');
      $usernameToken = $domRequest->createElementNS(self::$WssNamespace, 'wsse:UsernameToken');
      $username = $domRequest->createElementNS(self::$WssNamespace, 'wsse:Username', $this->login);
      $password = $domRequest->createElementNS(self::$WssNamespace, 'wsse:Password', $this->password);
      $usernameToken->appendChild($username);
      $usernameToken->appendChild($password);
      $securityToken->appendChild($usernameToken);
      $header->appendChild($securityToken);
    }

    // Do request
    $request = $domRequest->saveXML();
    $response = parent::__doRequest($request, $location, $action, $version, $one_way);

    // Extract SOAP XML from multipart message
    $response = $this->__extractSoapEnvelope($response);
    return $response;
  }

  /**
   * Extract the SOAP message from a multipart message.
   */
  private function __extractSoapEnvelope($message) {
    $matches = array();
    $result = preg_match('#<soap:Envelope(?:\s+[^>]+)?>(.*?)</soap:Envelope>#s', $message, $matches);
    if ($result) {
      $message = '<?xml version="1.0" encoding="UTF-8"?>' . $matches[0];
    }
    return $message;
  }
}